#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-data-single-catalog-config v. 2.1.2
#
# Post-installation script for configuring the gLite Data Management Single Catalog Node
# Robert Harakaly  < robert.harakaly@cern.ch >
#
# Version info: $Id: glite-data-single-catalog-config.py,v 1.41.2.8 2006/01/22 19:21:33 dimeglio Exp $
#
# Usage: python glite-data-single-catalog-config [-c|-v|-h|-siteconfig=<site config url>|--start|--stop|--status|--configure|--help]
#        -c                             print configuration
#        -v                             print version
#        -h,--help                      print usage info
#        -siteconfig=<site config url>  use site configuration file
#        --start                        start the service
#        --stop                         start the service
#        --db_force_create              force DB recreation. Data will be lost. 
#        --status                       shows the status of the service
#        --configure                    configures the service
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,socket,time
import sys
import getopt
import commands
import shutil

from  gLiteInstallerLib import gLib
from  gLiteInstallerLib import ConfigParams
from  gridSecurity import gridSecurity
from mysql import Mysql
from tomcat5 import Secure_tomcat5
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params           # all config values from the XML file
global mysql            # the one and only mysql service instance
global db_force_create  # if set to true, it will recreate the DB and data will be lost
global verbose

class glite_data_single_catalog:

    def __init__(self):
        self.verbose = 0
        self.version = "2.1.2"
        self.name = "glite-data-single-catalog"
        self.friendly_name = "Data Management Single Catalog Service"
        self.glib = params['gLib']
        self.mysql = Mysql()
        self.tomcat = Secure_tomcat5(params)
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    

    #################################################################
    # usage 
    #
    # print usage
    #
    #################################################################
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()

        print "Usage: \n"
        print "Edit the configuration file %s.cfg.xml in " %  self.name,
        print "%s/etc.config/templates" % os.environ['GLITE_LOCATION']
        print "save it as %s/etc/config/%s.cfg.xml" % (os.environ['GLITE_LOCATION'], self.name),
        print "and run the script as follows"
        print "python %s-config [OPTION...]" % self.name
        print ''
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure         configure %s' % self.friendly_name
        print '    --start             start %s' % self.friendly_name
        print '    --stop              stop %s' % self.friendly_name
        print '    --db_force_create   force DB recreation. Data will be lost.'
        print '    --status            print service status (0 = running, 1 = idle)'
        print '    --siteconfig=URL    use site configuration file'
        print '\n'


    #################################################################
    # configure
    #
    # configure data single catalog (MySQL)
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configure(self):

        #--------------------------------------------------------
        # Check java installation
        #--------------------------------------------------------
        if glib.checkJava() != 0:
            return 1

        #-------------------------------------------------------------------------------
        # VO list notice
        #-------------------------------------------------------------------------------
    
        print "\nThis Catalog will be configured to serve the following VOs:"
        for vo in glib.getVoList():
            print "=> %s" % vo
            
        #--------------------------------------------------------
        # Configure the Security Utilities
        #--------------------------------------------------------
        
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "Configuring gLite security                                ",
            glib.printFailedMessage()
            return 1

        print "Configuring gLite security                                ",
        glib.printOkMessage()
            
        #-------------------------------------------------------------------------------
        # Create vomsdir and check for VOMS certificates
        #-------------------------------------------------------------------------------

        if gsec.createVomsdir():
            return 1

        #--------------------------------------------------------
	    # Check configuration values
	    #--------------------------------------------------------
		
        print "Checking configuration values                             ",		

        # httpconnector_maxThreads
        if params.has_key('catalog-service-fr-mysql.httpconnector_maxThreads'):
            httpconnector_maxThreads = params['catalog-service-fr-mysql.httpconnector_maxThreads']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"catalog-service-fr-mysql.httpconnector_maxThreads\" not found.")
            return 1

        # httpconnector_minSpareThreads
        if params.has_key('catalog-service-fr-mysql.httpconnector_minSpareThreads'):
            httpconnector_minSpareThreads = params['catalog-service-fr-mysql.httpconnector_minSpareThreads']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"catalog-service-fr-mysql.httpconnector_minSpareThreads\" not found.")
            return 1
			
        # httpconnector_maxSpareThreads
        if params.has_key('catalog-service-fr-mysql.httpconnector_maxSpareThreads'):
            httpconnector_maxSpareThreads = params['catalog-service-fr-mysql.httpconnector_maxSpareThreads']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"catalog-service-fr-mysql.httpconnector_maxSpareThreads\" not found.")
            return 1

        # httpconnector_acceptCount
        if params.has_key('catalog-service-fr-mysql.httpconnector_acceptCount'):
            httpconnector_acceptCount = params['catalog-service-fr-mysql.httpconnector_acceptCount']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"catalog-service-fr-mysql.httpconnector_acceptCount\" not found.")
            return 1
		
        # httpconnector_connectionTimeout
        if params.has_key('catalog-service-fr-mysql.httpconnector_connectionTimeout'):
            httpconnector_connectionTimeout = params['catalog-service-fr-mysql.httpconnector_connectionTimeout']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"catalog-service-fr-mysql.httpconnector_connectionTimeout\" not found.")
            return 1	
		
        # mysql root password
        if params.has_key('mysql.root.password'):
        	mysql_root_password = params['mysql.root.password']
        else:
            mysql_root_password = ""	

		# allow unsecure port
        if params.has_key('allow.unsecure.port'):
            allow_unsecure_port = params['allow.unsecure.port']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"allow.unsecure.port\" not found.")
            return 1

        glib.printOkMessage()
        
        #--------------------------------------------------------
        # Configure Tomcat
        #--------------------------------------------------------
        
        if not params.has_key('CATALINA_HOME'):
            if self.glib.check_existing("/var/lib/tomcat5"):
                glib.printErrorMessage("Cannot find tomcat")
                return 1
            else:
                glib.export("CATALINA_HOME","/var/lib/tomcat5") 

        if self.glib.check_existing("%s/conf/Catalina/localhost" % params['CATALINA_HOME']):
            glib.printErrorMessage("Tomcat webapps config directory does not exist")
            return 1

        if self.tomcat.configure() != 0:
            return 1
            		
        print "Configuring Tomcat connector ports                        ",
		
        # configure ports
        securePort   = '8443'
        unsecurePort = '8080'
        errorcode = 0
        errorcode += self.tomcat.setConnectorParameter(securePort, "maxThreads", httpconnector_maxThreads)
        errorcode += self.tomcat.setConnectorParameter(securePort, "minSpareThreads", httpconnector_minSpareThreads)
        errorcode += self.tomcat.setConnectorParameter(securePort, "maxSpareThreads", httpconnector_maxSpareThreads)
        errorcode += self.tomcat.setConnectorParameter(securePort, "acceptCount", httpconnector_acceptCount)
        errorcode += self.tomcat.setConnectorParameter(securePort, "connectionTimeout", httpconnector_connectionTimeout)
		
        if allow_unsecure_port=="true":
            errorcode += self.tomcat.setConnectorParameter(unsecurePort, "maxThreads", httpconnector_maxThreads)
            errorcode += self.tomcat.setConnectorParameter(unsecurePort, "minSpareThreads", httpconnector_minSpareThreads)
            errorcode += self.tomcat.setConnectorParameter(unsecurePort, "maxSpareThreads", httpconnector_maxSpareThreads)
            errorcode += self.tomcat.setConnectorParameter(unsecurePort, "acceptCount", httpconnector_acceptCount)
            errorcode += self.tomcat.setConnectorParameter(unsecurePort, "connectionTimeout", httpconnector_connectionTimeout)

        if errorcode:
        	glib.printFailedMessage()
        	return 1
        	
        glib.printOkMessage()
            
        #--------------------------------------------------------
        # Configure MySQL
        #--------------------------------------------------------

        # start MySQL
        self.mysql.start()                

        # ------------------------------------------------------------
        # Check password of MySQL
        # ------------------------------------------------------------
        
        if not params.has_key('set.mysql.root.password'):
            params['set.mysql.root.password'] = 'false'
        setempty = params['set.mysql.root.password']
        if self.mysql.checkMySQLConfiguration(mysql_root_password,setempty):
            return 1
        
        #--------------------------------------------------------
        # Configure all instances
        #--------------------------------------------------------
        instances = self.glib.getInstances('FiremanCatalog')
        if len(instances) >= 0:
            for name in instances.keys():
                self.configureInstance(name)

        #-------------------------------------------------------------------
        # RGMA servicetool: configure servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":

            # Instantiate the rgma-servicetool class
            rgmaServicetool = gliteRgmaServicetool()
            rgmaServicetool.verbose = self.verbose
            
            # Create Fireman Catalog instances
            for vo in glib.getVoList():
                
                serviceId = glib.fq_hostname + "_" + vo + "_" + params['firemanCatalog.serviceType']
                servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
                
                # set params
                servicetoolInstance.setServiceName('MySQL Fireman Catalog service for %s at %s' % (vo,glib.fq_hostname))
                servicetoolInstance.setServiceType(params['firemanCatalog.serviceType'])
                servicetoolInstance.setServiceVersion(self.version)
                servicetoolInstance.setStatusScript('wget -S -r https://%s:8443/%s/glite-data-catalog-service-fr-mysql/services/FiremanCatalog' % (glib.fq_hostname,vo))
                servicetoolInstance.setEndpoint('https://%s:8443/%s/glite-data-catalog-service-fr-mysql/services/FiremanCatalog' % (glib.fq_hostname,vo))
                servicetoolInstance.setWsdlUrl('https://%s:8443/%s/glite-data-catalog-service-fr-mysql/services/FiremanCatalog?wsdl' % (glib.fq_hostname,vo))
                # add instance to the gLite configuration
                if servicetoolInstance.add() == 1:
                    return 1
                
                # Create SEIndex instance
                serviceId = glib.fq_hostname + "_" + vo + "_" + params['seIndex.serviceType']
                servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
                
                # set params
                servicetoolInstance.setServiceName('MySQL SE Index service for %s at %s' % (vo,glib.fq_hostname))
                servicetoolInstance.setServiceType(params['seIndex.serviceType'])
                servicetoolInstance.setServiceVersion(self.version)
                servicetoolInstance.setStatusScript('wget -S -r https://%s:8443/%s/glite-data-catalog-service-fr-mysql/services/SEIndex' % (glib.fq_hostname,vo))
                servicetoolInstance.setEndpoint('https://%s:8443/%s/glite-data-catalog-service-fr-mysql/services/SEIndex' % (glib.fq_hostname,vo))
                servicetoolInstance.setWsdlUrl('https://%s:8443/%s/glite-data-catalog-service-fr-mysql/services/SEIndex?wsdl' % (glib.fq_hostname,vo))
                
                # add instance to the gLite configuration
                if servicetoolInstance.add() == 1:
                    return 1
                
            # Configure servicetool
            if rgmaServicetool.configure(glib):
                # error in configuring servicetool
                return 1
        
        return 0


    #################################################################
    # configureInstance
    #
    # configure one (VO) instance
    #
    # return 0 ok
    #        1 error
    #
    #################################################################    
    def configureInstance(self, name='default'):

        instanceParams = self.glib.getInstanceParams(name)
        if name == 'default':
            name = 'glite-data-catalog-service-fr-mysql'

        if not instanceParams.has_key('catalog-service-fr-mysql.DBPASSWORD'):
            instanceParams['catalog-service-fr-mysql.DBPASSWORD'] = ""
        
        #--------------------------------------------------------
        # Create DB if it doesn't exists or self.db_force_create is true
        #--------------------------------------------------------
#        if self.mysql.existsDB(instanceParams['catalog-service-fr-mysql.DBNAME'],params['mysql.root.password']) == 1 or \
#            params['db.force.create'] == 'true':
        if self.mysql.existsDB(instanceParams['catalog-service-fr-mysql.DBNAME'],params['mysql.root.password']) == 1 or \
            db_force_create == 'true':

            
            #--------------------------------------------------------
            #  Create DB user
            #--------------------------------------------------------
            self.mysql.add_user(instanceParams['catalog-service-fr-mysql.DBNAME'], instanceParams['catalog-service-fr-mysql.DBUSER'],\
                    instanceParams['catalog-service-fr-mysql.DBPASSWORD'],params['mysql.root.password'])
            
            #--------------------------------------------------------
            #  load schema
            #--------------------------------------------------------
            os.system("mysql -u%s -p%s %s < %s/etc/glite-data-catalog-service-fr-mysql/schema.sql" \
                %(instanceParams['catalog-service-fr-mysql.DBUSER'], instanceParams['catalog-service-fr-mysql.DBPASSWORD'],\
                instanceParams['catalog-service-fr-mysql.DBNAME'], os.environ['GLITE_LOCATION']))
                
        else:
            glib.printInfoMessage("The %s database already exists and will not be recreated" % instanceParams['catalog-service-fr-mysql.DBNAME'])
            glib.printInfoMessage("Set the db.force.create parameter to true in the configuration file")
            glib.printInfoMessage("to force dropping and recreating the database\n")

        #--------------------------------------------------------
        #  Replace tokens in the context.xml template file
        #--------------------------------------------------------
        warpath = "%s/webapps/%s/glite-data-catalog-service-fr-mysql" \
                % (os.environ["CATALINA_HOME"],instanceParams['vo.name'])
        if os.path.isdir(warpath):
                shutil.rmtree(warpath)
                print "Removed existing web application from ", warpath, "\n"

        inFile = open("%s/etc/glite-data-catalog-service-fr-mysql/context.xml.template" % os.environ['GLITE_LOCATION'],"r")
        outFile = open("%s/conf/Catalina/localhost/glite-data-catalog-service-fr-mysql.%s.xml" % \
            (os.environ["CATALINA_HOME"], name.split(':')[1]) ,"w")

        for line in inFile.readlines():
            line = line.replace("@PATH@", "/%s/glite-data-catalog-service-fr-mysql" % instanceParams['vo.name'])
            line = replaceTags(line, instanceParams, 'catalog-service-fr-mysql')
            outFile.write(line)

        outFile.close()
        inFile.close()

        os.system("cp %s/lib/glite-data-common/*.jar /usr/share/tomcat5/common/lib/" \
            % os.environ['GLITE_LOCATION'])   

        return 0
    
    
    #################################################################
    # start
    #
    # start SC MySQL
    #
    # return 0 ok
    #        1 error
    #
    #################################################################    
    def start(self):

        if len(os.listdir(params['X509_VOMS_DIR'])) == 0:
            print("\n\n")
            glib.printErrorMessage("No VOMS Server host certificates could be found in %s" % params['X509_VOMS_DIR'])
            glib.printErrorMessage("Before using the %s," % service.friendly_name)
            glib.printErrorMessage("make sure that the host certificate of each used VOMS")
            glib.printErrorMessage("Server is installed in the %s directory." % params['X509_VOMS_DIR'])
            return 1

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "      Starting the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
 
        #-------------------------------------------------------------------
        # Start MySQL
        #-------------------------------------------------------------------
        
        # check if MySQL is already running
        if os.path.exists("/var/lock/subsys/mysql"):
            glib.printInfoMessage("MySQL is already running. Stopping and restarting MySQL ...")
            errorcode = self.mysql.stop()
            if errorcode:
                return 1
		
		# start MySQL	
        self.mysql.start()

        #-------------------------------------------------------------------
        # Start Tomcat
        #-------------------------------------------------------------------

        if self.tomcat.conditionalStart() != 0:
            return 1
        
        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------
    
        if params['rgma.servicetool.activate'] == "true":
    
            errorcode = rgmaServicetool.start()
            if (errorcode != 0):
                return 1
        
        return 0
    
    
    #################################################################
    # stop
    #
    # stop SC MySQL
    #
    # return 0 ok
    #        1 error
    #
    #################################################################    
    def stop(self):
    	
        error_level = 0
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "        Stopping the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        #-------------------------------------------------------------------
        # Stop tomcat
        #-------------------------------------------------------------------
    	
        if self.tomcat.stop() != 0:
            error_level = 1
        
        #-------------------------------------------------------------------
        # Stop MySQL
        #-------------------------------------------------------------------
        
        if self.mysql.stop() != 0:
            error_level = 1
        
        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":

            if rgmaServicetool.stop():
                error_level = 1
    
        return error_level


    #################################################################
    # status
    #
    # print status of catalog
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def status(self):
    	
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "    Checking status of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        errorcode = 0
        
        if params.has_key('host.certificate.file'):
            hostcert_file = params['host.certificate.file']
        else:
            hostcert_file = "/etc/grid-security/hostcert.pem"
        
        if params.has_key('host.key.file'):	
            hostkey_file  = params['host.key.file']
        else:
            hostkey_file = "/etc/grid-security/hostkey.pem"
            
        # check for each VO:
        instances = self.glib.getInstances('FiremanCatalog')
        if len(instances) > 0:

            for name in instances.keys():

            	instanceParams = self.glib.getInstanceParams(name)
        
                # get VO name
                if instanceParams.has_key('vo.name'):
            	    vo_name = instanceParams['vo.name']
            	    
                    if self.verbose:
                        print "Checking status of VO '%s': " % vo_name,
                    status = self.glib.checkHTTPSconnection("localhost", "/%s/glite-data-catalog-service-fr-mysql/services/FiremanCatalog?method=getVersion" 
                                % vo_name, 8443, hostkey_file, hostcert_file)
                    
                    if status[0]:
                        if self.verbose:
        		            print "not running. (%s)" % status[2]
                        errorcode = 1
                    else:
                        print("running.")

        if errorcode:
            print "\n%s is not running." % self.friendly_name
        else:
            print "\n%s is running." % self.friendly_name
            
        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":

            if rgmaServicetool.status() != 0:
                errorcode = 1
    
        return errorcode

#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    params['mysql.root.password'] = ""
    
    params['firemanCatalog.serviceType'] = 'org.glite.FiremanCatalog'
    params['seIndex.serviceType'] = 'org.glite.SEIndex'
    
#################################################################
# set_env
#
# Set all environment variables 
#
#################################################################
def set_env():
    
    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    if not params.has_key('X509_VOMS_DIR'):
        params['X509_VOMS_DIR'] = '/etc/grid-security/vomsdir'
    glib.export('X509_VOMS_DIR', params['X509_VOMS_DIR'])

    glib.export("JAVA_HOME")
    glib.export("CATALINA_HOME","/var/lib/tomcat5")

    glib.addEnvPath("PATH", "%s/bin" % os.environ["JAVA_HOME"])
    glib.addEnvPath("PATH", "%s/externals/bin" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("PATH", "%s/bin" % os.environ["GLITE_LOCATION"])

    glib.addEnvPath("LD_LIBRARY_PATH", "%s/externals/lib" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ["GLITE_LOCATION"])
    
    # MANPATH
    glib.addEnvPath('MANPATH', os.environ['GLITE_LOCATION'] + '/share/man')

    # Set environment
    glib.setUserEnv()


#################################################################
# successfulConfig
#
# Stop services after a successful configuration
#
#################################################################
def successfulConfig():

    # Stop all services
    glib.printInfoMessage("Stopping all running Fireman services...")
    service.stop()

    glib.registerService()

    print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
    print "\nBefore using the %s services make sure that" % service.friendly_name
    print "the host certificate of each used VOMS server is installed in"
    print "%s\n" % params['X509_VOMS_DIR']
    print "You can now start the service using the --start option of this script\n\n"
    sys.exit(0)


#################################################################
# warningsConfig
#
# Stop services and display message due to warnings found during configuration 
#
#################################################################
def warningsConfig():

    glib.printInfoMessage("Stopping all running Fireman services...")
    service.stop()

    print "\n\nThe %s configuration was completed, but one or more warnings" % service.friendly_name
    print "were issued. Please revise them and re-run the script"
    print "or configure and start the %s manually\n" % service.friendly_name
    sys.exit(2)


#################################################################
# replaceTags
#
# replace tags in file
#
# return 0 ok
#        1 error
#
#################################################################     
def replaceTags(line, params, prefix):
    if line.find("@") > -1:
        tags = line.split("@")
        i = 1
        while i < len(tags):
            if params.has_key(prefix + "." + tags[i]):
                paramName = "%s.%s" % (prefix, tags[i])
                if verbose:
                    print("Replace @%s@ by %s" % (tags[i], params[paramName]))
                line = line.replace("@%s@" % tags[i], params[paramName])
            else:
                if verbose:
                    print("Replace @%s@ by empty string" %(tags[i]))
                line = line.replace("@%s@" % tags[i], "")
            i += 2
    return line

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    
    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass

    if glib.loadConfiguration(["%s/../glite-data-single-catalog.cfg.xml" % glib.getScriptPath(), "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()],params):
        print "An error occurred while configuring the service"
        sys.exit(1)
    params['gLib'] = glib
    
    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------	
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose

    # ------------------------------------------------------------
    # set db_force_create
    # ------------------------------------------------------------
    db_force_create = "false"
 
    # ------------------------------------------------------------
    # Set up the environment
    # ------------------------------------------------------------
    set_env()
          
    #Instantiate grdSecurity classes
    gsec = gridSecurity(glib)
    gsec.setVerbosity(verbose)
    gsec.params = params

    # ------------------------------------------------------------	      
    # Instantiate the service classes
    # ------------------------------------------------------------
    service = glite_data_single_catalog()
    service.verbose = verbose
    
    # Instantiate the rgma servicetool class
    if params['rgma.servicetool.activate'] == 'true':
        rgmaServicetool = gliteRgmaServicetool()
        rgmaServicetool.verbose = verbose

    # ------------------------------------------------------------
    # Command line opts if any
    # ------------------------------------------------------------
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'start', 'stop', 'status', 'db_force_create', 'configure', 'siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # ------------------------------------------------------------
    # Check cli options
    # ------------------------------------------------------------	
    for o, a in opts:
        
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
            
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
            
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)
            
        if o == "--configure":
            # Configure the service without recreating the DB if already exists
            #--------------------------------------------------------
            # Check certificates
            #--------------------------------------------------------
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)

            service.copyright()
            service.showVersion()
            service.banner()

            #--------------------------------------------------------
            # Display the parameters if verbose output
            #--------------------------------------------------------
            if verbose:
                glib.print_params(params)

            #--------------------------------------------------------
            # Stop all services
            #--------------------------------------------------------
            glib.printInfoMessage("Stopping all running Fireman services...")
            service.stop()

            return_value = service.configure()
            
            if return_value == 0:
                # Stop services after a successful configuration
                successfulConfig()
            elif return_value == 2:
                # Stop services and display message due to warnings found during configuration
                warningsConfig()
            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name
                sys.exit(1)
            
        if o == "--start":
            if service.start() == 0:
                print "\n\nThe %s were successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while starting the %s" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--stop":
            if service.stop() == 0:
                print "\n\nThe %s were successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while stopping the %s" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--db_force_create":
            # Configure the service recreating the DB and loosing old data.
            #--------------------------------------------------------
            # Check certificates
            #--------------------------------------------------------
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)

            service.copyright()
            service.showVersion()
            service.banner()

            #--------------------------------------------------------
            # Display the parameters if verbose output
            #--------------------------------------------------------
            if verbose:
                glib.print_params(params)

            #--------------------------------------------------------
            # Stop all services
            #--------------------------------------------------------
            glib.printInfoMessage("Stopping all running Fireman services...")
            service.stop()

            db_force_create = "true" 
            return_value = service.configure()

            if return_value == 0:
                # Stop services after a successful configuration
                successfulConfig()
            elif return_value == 2:
                # Stop services and display message due to warnings found during configuration
                warningsConfig()
            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--status":
            errorcode = service.status()
            if errorcode:
                sys.exit(1)
            sys.exit(0)

